<?php defined('BASEPATH') OR exit('No direct script access allowed');

require APPPATH . '/libraries/REST_Controller.php';
use Restserver\Libraries\REST_Controller;

class Sales extends REST_Controller {

    function __construct() {
        parent::__construct();

        $this->methods['index_get']['limit'] = 500;
        $this->load->api_model('sales_api');
    }

    public function index_get() {
        $reference = $this->get('reference');

        $filters = [
        'reference' => $reference,
        'include' => $this->get('include') ? explode(',', $this->get('include')) : NULL,
        'start' => $this->get('start') && is_numeric($this->get('start')) ? $this->get('start') : 1,
        'limit' => $this->get('limit') && is_numeric($this->get('limit')) ? $this->get('limit') : 10,
        'start_date' => $this->get('start_date') && is_numeric($this->get('start_date')) ? $this->get('start_date') : NULL,
        'end_date' => $this->get('end_date') && is_numeric($this->get('end_date')) ? $this->get('end_date') : NULL,
        'order_by' => $this->get('order_by') ? explode(',', $this->get('order_by')) : ['id', 'decs'],
        'customer_id' => $this->get('customer_id') ? $this->get('customer_id') : NULL,
        'customer' => $this->get('customer') ? $this->get('customer') : NULL,
        ];

        if ($reference === NULL) {

            if ($sales = $this->sales_api->getSales($filters)) {
                $sl_data = [];
                foreach ($sales as $sale) {
                    if (!empty($filters['include'])) {
                        foreach ($filters['include'] as $include) {
                            if ($include == 'items') {
                                $sale->items = $this->sales_api->getSaleItems($sale->id);
                            }
                            if ($include == 'warehouse') {
                                $sale->warehouse = $this->sales_api->getWarehouseByID($sale->warehouse_id);
                            }
                        }
                    }

                    $sale->created_by = $this->sales_api->getUser($sale->created_by);
                    $sl_data[] = $this->setSale($sale);
                }

                $data =  [
                'data' => $sl_data,
                'limit' => (int) $filters['limit'],
                'start' => (int) $filters['start'],
                'total' => $this->sales_api->countSales($filters),
                ];
                $this->response($data, REST_Controller::HTTP_OK);

            } else {
                $this->response([
                    'message' => 'No sale record found.',
                    'status' => FALSE,
                    ], REST_Controller::HTTP_NOT_FOUND);
            }

        } else {

            if ($sale = $this->sales_api->getSale($filters)) {

                if (!empty($filters['include'])) {
                    foreach ($filters['include'] as $include) {
                        if ($include == 'items') {
                            $sale->items = $this->sales_api->getSaleItems($sale->id);
                        }
                        if ($include == 'warehouse') {
                            $sale->warehouse = $this->sales_api->getWarehouseByID($sale->warehouse_id);
                        }
                    }
                }

                $sale->created_by = $this->sales_api->getUser($sale->created_by);
                $sale = $this->setSale($sale);
                $this->set_response($sale, REST_Controller::HTTP_OK);

            } else {
                $this->set_response([
                    'message' => 'Sale could not be found for reference '.$reference.'.',
                    'status' => FALSE,
                    ], REST_Controller::HTTP_NOT_FOUND);
            }
        }
    }

    protected function setSale($sale) {
        unset($sale->address_id, $sale->api, $sale->attachment, $sale->hash, $sale->pos, $sale->reserve_id, $sale->return_id, $sale->return_sale_ref, $sale->return_sale_total, $sale->sale_id, $sale->shop, $sale->staff_note, $sale->surcharge, $sale->updated_at, $sale->suspend_note);
        if (isset($sale->items) && !empty($sale->items)) {
            foreach($sale->items as &$item) {
                if (isset($item->option_id) && !empty($item->option_id)) {
                    if($variant = $this->sales_api->getProductVariantByID($item->option_id)) {
                        $item->product_variant_id = $variant->id; 
                        $item->product_variant_name = $variant->name; 
                    }
                }
                $item->product_unit_quantity = $item->unit_quantity;
                unset($item->id, $item->sale_id, $item->warehouse_id, $item->real_unit_price, $item->sale_item_id, $item->option_id, $item->unit_quantity);
                $item = (array) $item;
                ksort($item);
            }
        }
        $sale = (array) $sale;
        ksort($sale);
        return $sale;
    }

}
